@@ -4,7 +4,7 @@ use 5.010;
 use strict;
 use warnings;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 use overload
   '""' => \&stringify,
@@ -85,7 +85,7 @@ use overload
         acsc  => \&acsc,
         acsch => \&acsch,
 
-        atan2 => \&atan2,
+        atan2 => sub ($$) { goto &atan2 },    # built-in function
 
         #deg2rad => \&deg2rad,
         #rad2deg => \&rad2deg,
@@ -127,9 +127,10 @@ use overload
 
             if ($name eq ':overload') {
                 overload::constant
-                  integer => sub { __PACKAGE__->new($_[0],      0) },
-                  float   => sub { __PACKAGE__->new($_[0],      0) },
-                  binary  => sub { __PACKAGE__->new(oct($_[0]), 0) };
+                  integer => sub { __PACKAGE__->new($_[0], 0) },
+                  float   => sub { __PACKAGE__->new($_[0], 0) };
+
+                #binary  => sub { __PACKAGE__->new(oct($_[0]), 0) };
 
                 # Export the 'i' constant
                 foreach my $pair (['i', i()]) {
@@ -189,21 +190,26 @@ sub new {
           }, $class;
 }
 
-#
-## i = sqrt(-1)
-#
-
-sub i {
-    __PACKAGE__->new(0, 1);
-}
-
 #
 ## cplx(a, b) = a + b*i
 #
 
 sub cplx {
     my ($x, $y) = @_;
-    __PACKAGE__->new($x, $y);
+
+    bless {
+           a => $x // 0,
+           b => $y // 0,
+          },
+      __PACKAGE__;
+}
+
+#
+## i = sqrt(-1)
+#
+
+sub i {
+    __PACKAGE__->new(0, 1);
 }
 
 #
@@ -257,6 +263,10 @@ sub div {
 
     my $d = $y->{a} * $y->{a} + $y->{b} * $y->{b};
 
+    if (!ref($d) and $d == 0) {
+        return $x->log->sub($y->log)->exp;
+    }
+
     __PACKAGE__->new(($x->{a} * $y->{a} + $x->{b} * $y->{b}) / $d, ($x->{b} * $y->{a} - $x->{a} * $y->{b}) / $d);
 }
 
@@ -274,17 +284,17 @@ sub mod {
 }
 
 #
-## inv(a + b*i) = a/(a^2 + b^2) - i*b/(a^2 + b^2)
+## inv(x) = 1/x
 #
 
-sub inv {
+sub inv ($) {
     my ($x) = @_;
 
     $x = __PACKAGE__->new($x) if ref($x) ne __PACKAGE__;
 
-    my $den = $x->{a} * $x->{a} + $x->{b} * $x->{b};
+    state $one = __PACKAGE__->new(1, 0);
 
-    __PACKAGE__->new($x->{a} / $den, -$x->{b} / $den);
+    $one->div($x);
 }
 
 #
@@ -303,7 +313,7 @@ sub abs {
 ## sgn(a + b*i) = (a + b*i) / abs(a + b*i)
 #
 
-sub sgn {
+sub sgn ($) {
     my ($x) = @_;
 
     $x = __PACKAGE__->new($x) if ref($x) ne __PACKAGE__;
@@ -331,7 +341,7 @@ sub neg {
 ## conj(a + b*i) = a - b*i
 #
 
-sub conj {
+sub conj ($) {
     my ($x) = @_;
 
     $x = __PACKAGE__->new($x) if ref($x) ne __PACKAGE__;
@@ -351,7 +361,7 @@ sub log {
     my $t = $x->{a} * $x->{a} + $x->{b} * $x->{b};
 
     if (!ref($t) and $t == 0) {
-        return __PACKAGE__->new(-'inf', 0);
+        return __PACKAGE__->new(0 + '-Inf', 0);
     }
 
     __PACKAGE__->new(CORE::log($t) / 2, CORE::atan2($x->{b}, $x->{a}));
@@ -361,7 +371,7 @@ sub log {
 ## logn(x, n) = log(x) / log(n)
 #
 
-sub logn {
+sub logn ($$) {
     my ($x, $n) = @_;
 
     $x = __PACKAGE__->new($x) if ref($x) ne __PACKAGE__;
@@ -401,7 +411,7 @@ sub pow {
 ## root(a + b*i, x) = (a + b*i)^(1/x)
 #
 
-sub root {
+sub root ($$) {
     my ($x, $y) = @_;
 
     $x = __PACKAGE__->new($x) if ref($x) ne __PACKAGE__;
@@ -431,7 +441,7 @@ sub sqrt {
 ## cbrt(a + b*i) = exp(log(a + b*i) / 3)
 #
 
-sub cbrt {
+sub cbrt ($) {
     my ($x) = @_;
 
     $x = __PACKAGE__->new($x) if ref($x) ne __PACKAGE__;
@@ -467,7 +477,7 @@ sub int {
 ## floor(a + b*i) = floor(a) + floor(b)*i
 #
 
-sub floor {
+sub floor ($) {
     my ($x) = @_;
 
     $x = __PACKAGE__->new($x) if ref($x) ne __PACKAGE__;
@@ -485,7 +495,7 @@ sub floor {
 ## ceil(a + b*i) = -floor(-(a + b*i))
 #
 
-sub ceil {
+sub ceil ($) {
     my ($x) = @_;
 
     $x = __PACKAGE__->new($x) if ref($x) ne __PACKAGE__;
@@ -529,7 +539,7 @@ sub sin {
 ## sinh(a + b*i) = (exp(2 * (a + b*i)) - 1) / (2*exp(a + b*i))
 #
 
-sub sinh {
+sub sinh ($) {
     my ($x) = @_;
 
     $x = __PACKAGE__->new($x) if ref($x) ne __PACKAGE__;
@@ -550,7 +560,7 @@ sub sinh {
 ## asin(a + b*i) = -i*log(sqrt(1 - (a + b*i)^2) + i*a - b)
 #
 
-sub asin {
+sub asin ($) {
     my ($x) = @_;
 
     $x = __PACKAGE__->new($x) if ref($x) ne __PACKAGE__;
@@ -569,7 +579,7 @@ sub asin {
 ## asinh(a + b*i) = log(sqrt((a + b*i)^2 + 1) + (a + b*i))
 #
 
-sub asinh {
+sub asinh ($) {
     my ($x) = @_;
 
     $x = __PACKAGE__->new($x) if ref($x) ne __PACKAGE__;
@@ -611,7 +621,7 @@ sub cos {
 ## cosh(a + b*i) = (exp(2 * (a + b*i)) + 1) / (2*exp(a + b*i))
 #
 
-sub cosh {
+sub cosh ($) {
     my ($x) = @_;
 
     $x = __PACKAGE__->new($x) if ref($x) ne __PACKAGE__;
@@ -632,7 +642,7 @@ sub cosh {
 ## acos(a + b*i) = -2*i*log(i*sqrt((1 - (a + b*i))/2) + sqrt((1 + (a + b*i))/2))
 #
 
-sub acos {
+sub acos ($) {
     my ($x) = @_;
 
     $x = __PACKAGE__->new($x) if ref($x) ne __PACKAGE__;
@@ -659,7 +669,7 @@ sub acos {
 ## acosh(a + b*i) = log((a + b*i) + sqrt((a + b*i) - 1) * sqrt((a + b*i) + 1))
 #
 
-sub acosh {
+sub acosh ($) {
     my ($x) = @_;
 
     $x = __PACKAGE__->new($x) if ref($x) ne __PACKAGE__;
@@ -683,7 +693,7 @@ sub acosh {
 ## tan(a + b*i) = (2*i)/(exp(2*i*(a + b*i)) + 1) - i
 #
 
-sub tan {
+sub tan ($) {
     my ($x) = @_;
 
     $x = __PACKAGE__->new($x) if ref($x) ne __PACKAGE__;
@@ -711,7 +721,7 @@ sub tan {
 ## tanh(a + b*i) = (exp(2 * (a + b*i)) - 1) / (exp(2 * (a + b*i)) + 1)
 #
 
-sub tanh {
+sub tanh ($) {
     my ($x) = @_;
 
     $x = __PACKAGE__->new($x) if ref($x) ne __PACKAGE__;
@@ -728,7 +738,7 @@ sub tanh {
 ## atan(a + b*i) = i * (log(1 - i*(a + b*i)) - log(1 + i*(a + b*i))) / 2
 #
 
-sub atan {
+sub atan ($) {
     my ($x) = @_;
 
     $x = __PACKAGE__->new($x) if ref($x) ne __PACKAGE__;
@@ -764,7 +774,7 @@ sub atan2 {
 ## atanh(a + b*i) = (log(1 + (a + b*i)) - log(1 - (a + b*i))) / 2
 #
 
-sub atanh {
+sub atanh ($) {
     my ($x) = @_;
 
     $x = __PACKAGE__->new($x) if ref($x) ne __PACKAGE__;
@@ -789,7 +799,7 @@ sub atanh {
 ## cot(a + b*i) = (2*i)/(exp(2*i*(a + b*i)) - 1) + i
 #
 
-sub cot {
+sub cot ($) {
     my ($x) = @_;
 
     $x = __PACKAGE__->new($x) if ref($x) ne __PACKAGE__;
@@ -817,7 +827,7 @@ sub cot {
 ## coth(a + b*i) = (exp(2 * (a + b*i)) + 1) / (exp(2 * (a + b*i)) - 1)
 #
 
-sub coth {
+sub coth ($) {
     my ($x) = @_;
 
     $x = __PACKAGE__->new($x) if ref($x) ne __PACKAGE__;
@@ -834,7 +844,7 @@ sub coth {
 ## acot(a + b*i) = atan(1/(a + b*i))
 #
 
-sub acot {
+sub acot ($) {
     my ($x) = @_;
 
     $x = __PACKAGE__->new($x) if ref($x) ne __PACKAGE__;
@@ -846,7 +856,7 @@ sub acot {
 ## acoth(a + b*i) = atanh(1 / (a + b*i))
 #
 
-sub acoth {
+sub acoth ($) {
     my ($x) = @_;
 
     $x = __PACKAGE__->new($x) if ref($x) ne __PACKAGE__;
@@ -862,7 +872,7 @@ sub acoth {
 ## sec(a + b*i) = 2/(exp(-i*(a + b*i)) + exp(i*(a + b*i)))
 #
 
-sub sec {
+sub sec ($) {
     my ($x) = @_;
 
     $x = __PACKAGE__->new($x) if ref($x) ne __PACKAGE__;
@@ -888,7 +898,7 @@ sub sec {
 ## asec(a + b*i) = acos(1/(a + b*i))
 #
 
-sub asec {
+sub asec ($) {
     my ($x) = @_;
 
     $x = __PACKAGE__->new($x) if ref($x) ne __PACKAGE__;
@@ -900,7 +910,7 @@ sub asec {
 ## sech(a + b*i) = (2 * exp(a + b*i)) / (exp(2 * (a + b*i)) + 1)
 #
 
-sub sech {
+sub sech ($) {
     my ($x) = @_;
 
     $x = __PACKAGE__->new($x) if ref($x) ne __PACKAGE__;
@@ -920,7 +930,7 @@ sub sech {
 ## asech(a + b*i) = acosh(1/(a + b*i))
 #
 
-sub asech {
+sub asech ($) {
     my ($x) = @_;
 
     $x = __PACKAGE__->new($x) if ref($x) ne __PACKAGE__;
@@ -936,7 +946,7 @@ sub asech {
 ## csc(a + b*i) = -(2*i)/(exp(-i * (a + b*i)) - exp(i * (a + b*i)))
 #
 
-sub csc {
+sub csc ($) {
     my ($x) = @_;
 
     $x = __PACKAGE__->new($x) if ref($x) ne __PACKAGE__;
@@ -964,7 +974,7 @@ sub csc {
 ## csch(a + b*i) = (2*exp(a + b*i)) / (exp(2 * (a + b*i)) - 1)
 #
 
-sub csch {
+sub csch ($) {
     my ($x) = @_;
 
     $x = __PACKAGE__->new($x) if ref($x) ne __PACKAGE__;
@@ -984,7 +994,7 @@ sub csch {
 ## acsc(a + b*i) = asin(1/(a + b*i))
 #
 
-sub acsc {
+sub acsc ($) {
     my ($x) = @_;
 
     $x = __PACKAGE__->new($x) if ref($x) ne __PACKAGE__;
@@ -996,7 +1006,7 @@ sub acsc {
 ## acsch(a + b*i) = asinh(1/(a + b*i))
 #
 
-sub acsch {
+sub acsch ($) {
     my ($x) = @_;
 
     $x = __PACKAGE__->new($x) if ref($x) ne __PACKAGE__;
@@ -1008,7 +1018,7 @@ sub acsch {
 ## real(a + b*i) = a
 #
 
-sub real {
+sub real ($) {
     my ($x) = @_;
 
     $x = __PACKAGE__->new($x) if ref($x) ne __PACKAGE__;
@@ -1020,7 +1030,7 @@ sub real {
 ## imag(a + b*i) = b
 #
 
-sub imag {
+sub imag ($) {
     my ($x) = @_;
 
     $x = __PACKAGE__->new($x) if ref($x) ne __PACKAGE__;
@@ -1032,7 +1042,7 @@ sub imag {
 ## reals(a + b*i) = (a, b)
 #
 
-sub reals {
+sub reals ($) {
     my ($x) = @_;
 
     $x = __PACKAGE__->new($x) if ref($x) ne __PACKAGE__;
@@ -1082,7 +1092,7 @@ sub cmp {
       or ($x->{b} <=> $y->{b});
 }
 
-sub acmp {
+sub acmp ($$) {
     my ($x, $y) = @_;
 
     $x = __PACKAGE__->new($x) if ref($x) ne __PACKAGE__;
